# -*- coding: utf-8 -*-
#rotate to IK  回転をIKで制御したいときにどうぞ
#161202 ツイスト機能削除、ワールドのアップベクターで制御するように割り切った。
#既知の問題　安定性確認中

import maya.cmds as cmds
import fnmatch

shapeList = ["Locator", "cube", "rombus", "pyramid",]


#ロックをまとめて！
def locks(node, t=False, r=False, s=False, v=False):
    if t:cmds.setAttr(node+'.t', lock=True)
    if r:cmds.setAttr(node+'.r', lock=True)
    if s:cmds.setAttr(node+'.s', lock=True)
    if v:cmds.setAttr(node+'.v', lock=True)

#指定ノードを親とした座標を入力し、ワールド座標をリストで返します。
def parentBasePos2world(parent, x, y, z):
    print("---------------parentBasePos2world-----------------")
    cmds.spaceLocator(n="dummyLocator_P")#ダミーロケーターを作る
    cmds.parentConstraint(parent, "dummyLocator_P", weight=1 )
    cmds.parentConstraint(parent, "dummyLocator_P", remove=True )

    cmds.spaceLocator(n="dummyLocator")#ダミーロケーターを作る
    cmds.parent( "dummyLocator", "dummyLocator_P")
    cmds.setAttr( "dummyLocator.t", x, y, z)#ダミーヌルの位置にxyzを入れる
    cmds.parent("dummyLocator", w=True)#ペアレント解除
                    
    #そのダミーヌルの座標を取得する
    dTrans = cmds.xform("dummyLocator", q=True, t=True, ws=True)
    vector = [ dTrans[0], dTrans[1], dTrans[2] ]
    cmds.delete("dummyLocator", "dummyLocator_P")
    print("---------------end-----------------")
    return vector
    
    
#メイン
def main():
    #選択したかどうかをチェックして取得しておく
    if len(cmds.ls(sl=True)) == 0:cmds.warning("Not selected")
    else:
        #準備
        sels = cmds.ls(sl=True)
        ikVect = cmds.floatFieldGrp("ik", q=True, v=True)#フィールドから数値を取得しておく
        upvList = cmds.floatFieldGrp("UpV", q=True, v=True)
        startF = cmds.playbackOptions(q=True,minTime=True)#タイムラインの数値を取得しておく
        endF = cmds.playbackOptions(q=True,maxTime=True)
        ctrls = []
        #jointとハンドルをいれるグループを用意
        if cmds.objExists("rotate2ik_joints"):pass
        else:cmds.group(n="rotate2ik_joints", em=True)
        
        #一つずつ処理をする
        for sel in sels:
            print(u"処理をします = " + sel)
            try:
                selTrans = cmds.xform(sel, q=True, rp=True, ws=True)#選択ノードのワールド座標を取得
                cmds.select(cl=True)
                
                #名前を用意
                joint1 = sel+"_joint_r2ik1"
                joint2 = sel+"_joint_r2ik2"
                handle = sel+"_handle_r2ik"
                ctrl = sel+"_r2ik"

                
                #座標基準を元にik設置の座標を取得する 変数vectorに格納
                if cmds.radioButtonGrp("ikBase", q=True, select=True) == 1:#Positionだったら
                    vector = [  selTrans[0]+ikVect[0], selTrans[1]+ikVect[1], selTrans[2]+ikVect[2]  ]    
                elif cmds.radioButtonGrp("ikBase", q=True, select=True) == 2:#Parentなら
                    vector = parentBasePos2world(sel, ikVect[0], ikVect[1], ikVect[2])
                    print(vector)
                else:#ワールド座標なら
                    vector = ikVect

                #ジョイントを作る
                cmds.joint(n=joint1, p=selTrans )
                cmds.joint(  n=joint2, p=[  vector[0], vector[1], vector[2]  ]  )   
                cmds.joint(joint1, e=True, orientJoint="xyz", zso=True, sao ="yup")


                #選択ノードにコンストレイントポジション
                cmds.pointConstraint(sel, joint1)

                #コントローラーを作る
                size = cmds.floatField("ctrlsize",  q=True, v=True)
                curvetype = cmds.optionMenu('ShapeType', q=True, v=True)
                createCurve(curvetype, ctrl, size)
                locks(ctrl, s=True, v=True)
                
                #############もしアップベクターを追加するなら
                if cmds.checkBox("createUpV", q=True, v=True):
                    #アップベクターコントローラーを作り、選択ノードにペアレントコンスト
                    size = cmds.floatField("ctrlsize2",  q=True, v=True)
                    curvetype = cmds.optionMenu('ShapeType2', q=True, v=True)
                    createCurve(curvetype, sel+"_upv", size)
                    locks(sel+"_upv", s=True, v=True)
                    upv_const = cmds.parentConstraint(sel, sel+"_upv", mo=False)
                    if ":" in sel:
                        targetOffset = sel.split(":")[1]+"_upv_parentConstraint1.target[0].targetOffsetTranslate"
                    else:
                        targetOffset = sel+"_upv_parentConstraint1.target[0].targetOffsetTranslate"
                        
                    cmds.setAttr(targetOffset+"X", upvList[0] )
                    cmds.setAttr(targetOffset+"Y", upvList[1] )
                    cmds.setAttr(targetOffset+"Z", upvList[2] )
                else:pass    
                
                #joint2の位置にIKコントローラーを配置し、選択ノードにペアレントコンストをかける
                cmds.pointConstraint(joint2, ctrl)
                cmds.pointConstraint(joint2, ctrl, e=True, remove=True)
                cons = cmds.parentConstraint(sel, ctrl, st="none", mo=True)
                
                #コントローラーやアップベクターなどをベイク。作ったコンストは削除
                if cmds.checkBox("createUpV", q=True, v=True):
                    attList = [ctrl+".tx", ctrl+".ty", ctrl+".tz",   sel+"_upv.tx", sel+"_upv.ty", sel+"_upv.tz"]
                    cmds.bakeResults(attList, t=(startF, endF))
                    cmds.delete(cons, upv_const)
                    
                else:
                    attList = [ctrl+".tx", ctrl+".ty", ctrl+".tz"]
                    cmds.bakeResults(attList, t=(startF, endF))
                    cmds.delete(cons)
                
                #IKハンドルを作って、コントローラーに位置コンスト
                cmds.ikHandle(n=handle, sj=joint1, ee=joint2, sol="ikRPsolver")
                cmds.pointConstraint(ctrl, handle)
               
                #######もしアップベクターを追加するなら、IKハンドルとアップベクターロケーターをポールベクターコンストする
                if cmds.checkBox("createUpV", q=True, v=True):
                    cmds.poleVectorConstraint(sel+"_upv", handle)
                    
                #選択ノードとスケルトンをコンストレイント      
                if cmds.checkBox("maintainOffset", q=True, v=True):cmds.orientConstraint(joint1, sel, mo=True)
                else:cmds.orientConstraint(joint1, sel, mo=False)
                
                #作ったスケルトンとハンドルを格納する。邪魔なので
                cmds.parent(joint1,handle, "rotate2ik_joints")

                #コントローラーをリストに入れる
                ctrls.append(ctrl)
                if cmds.checkBox("createUpV", q=True, v=True):
                    ctrls.append(sel+"_upv")

            except:pass
        cmds.select(ctrls)

def returnIK():
    startF = cmds.playbackOptions(q=True,minTime=True)
    endF = cmds.playbackOptions(q=True,maxTime=True)
    
    for sel in cmds.ls(sl=True):
        if sel.count("_r2ik"):
            #選択したものから、制御されているノードを逆検索
            handle_consts =  cmds.listConnections(sel)
            sel_handle = cmds.listRelatives(fnmatch.filter(handle_consts,"*_handle_*")[0], path=True, parent=True)[0]
            r2ik_joint =  cmds.listConnections(sel_handle, type="joint")
            orientCons =  cmds.listConnections(r2ik_joint,type="orientConstraint")[0]
            returnTarget = cmds.listRelatives(orientCons, path=True, parent=True)[0]
            cmds.bakeResults(returnTarget+'.r',  t=(startF,endF), simulation=False )
             
            cmds.delete(sel,  r2ik_joint)
            
            upv = sel.replace("_r2ik", "_upv")
            if cmds.objExists(upv):
                print(upv)
                cmds.delete(upv)
            #もし"rotate2ik_joints"の中身が空っぽだったら削除
            if cmds.listRelatives("rotate2ik_joints", c=True)==None:
                cmds.delete("rotate2ik_joints")
                
#コントローラーの形        
def createCurve(curvetype,name,size):
    a = 1*size
    b = -1*size
    if curvetype == "Locator": 
        cmds.spaceLocator(n=name)
       
    elif curvetype == "cube":
        cmds.curve(n=name, d=1, p=[(a, a, a), (a, a, b), (b, a, b), (b, b, b), (a, b, b,),
        (a, a, b), (b, a, b), (b, a, a), (a, a, a), (a, b, a), (a, b, b), (b, b, b),
         (b, b, a), (a, b, a), (b, b, a), (b, a, a)])

    elif curvetype == "rombus":
        cmds.curve(n=name,d=1, p=[(0, a, 0), (a, 0, 0), (0, 0, a), (b, 0, 0), (0, 0, b), (0, a, 0), 
        (0, 0, a), (0, b, 0), (0, 0, b), (a, 0, 0), (0, a, 0), (b, 0, 0), (0, b, 0), (a, 0, 0)])

    elif curvetype == "pyramid":
        cmds.curve(n=name,d=1, p=[(0, a*2, 0), (a, 0, b), (b, 0, b), (0, a*2, 0), 
        (b, 0, a), (a, 0, a), (0, a*2, 0), (a, 0, b), (a, 0, a), (b, 0, a), (b, 0, b)]) 
          
    else:pass
    
    return name
    
#UI
class MoxRotatetoIK(object):
    def __init__(self):
        if cmds.window('MoxRotatetoIK', exists=True):
            cmds.deleteUI('MoxRotatetoIK')
            
        cmds.window('MoxRotatetoIK', menuBarVisible=True, tlb=True, s=1, t="Mox Rotate to IK", resizeToFitChildren=1, mnb=1, mxb=0)
        cmds.columnLayout('MainColumnLayout', adjustableColumn=True, rowSpacing=5, columnAttach=["both",5])
        
        cmds.checkBox("maintainOffset", l="maintainOffset", v=True, ann=u"スケルトンとターゲットのコンストレイントのオフセットを保持します。", visible=False)
        cmds.radioButtonGrp("ikBase", l="ik Base",numberOfRadioButtons=3, select =2, labelArray3 =["position", "parent", "world"], ann=u"Aim XYZの位置の基準を指定します", cw4=[45,70,70,70],visible=False)                      
        cmds.floatFieldGrp("ik",  numberOfFields= 3, label="ik XYZ" , value1 =0, value2=-30, value3=0,  cw4=[50,50,50,50])
        
        cmds.rowLayout(nc=4)
        cmds.text("shape")    
        shapes = cmds.optionMenu('ShapeType', w=80)
        for shape in shapeList:
            cmds.menuItem( label=shape)
        cmds.optionMenu('ShapeType', e=True, sl=1)
        cmds.text("ctrlsize2", l="      size")
        cmds.floatField("ctrlsize",  v=10, w=40, pre=1, step=0.1)
        cmds.setParent("..")
        
        cmds.separator()
        
        cmds.checkBox("createUpV", l="create UpVector the world",v=1, ann=u"アップベクター用のコントローラーを追加します。")
        cmds.floatFieldGrp("UpV",  numberOfFields= 3, label="upV XYZ" , value1 =0, value2=0, value3=-30,  cw4=[50,50,50,50])
        
        cmds.rowLayout(nc=4)
        cmds.text("shape")    
        shapes = cmds.optionMenu('ShapeType2', w=80)
        for shape in shapeList:
            cmds.menuItem( label=shape)
        cmds.optionMenu('ShapeType2', e=True, sl=1)
        cmds.text("ctrlsize2", l="      size")
        cmds.floatField("ctrlsize2",  v=10, w=40, pre=1, step=0.1)
        cmds.setParent("..")
        
        cmds.separator()
        cmds.rowLayout(nc=2, adjustableColumn=1)
        cmds.button(l="IK_Create", c='MoxRotatetoIK.main()' )
        cmds.button(l="Return", c='MoxRotatetoIK.returnIK()', bgc=[0.8, 0.8, 0.8] )
        cmds.setParent('..')
        cmds.showWindow('MoxRotatetoIK')
mr2ik_mainWind = MoxRotatetoIK()
